@@ -1,30 +1,31 @@
 mod ffi;
+#[cfg(test)]
+mod test;
+mod udbg;
 mod util;
-mod window;
 
-pub mod eng;
+#[cfg(feature = "dbgeng")]
+pub mod dbgeng;
 pub mod ntdll;
 pub mod string;
 pub mod symbol;
-pub mod udbg;
 
+pub use self::udbg::*;
 pub use self::util::*;
-pub use self::window::*;
 
-pub use self::eng::DebugEngine;
-pub use self::udbg::DefaultEngine;
+// see https://docs.microsoft.com/en-us/windows/win32/api/processthreadsapi/nf-processthreadsapi-getthreadpriority#return-value
+pub type priority_t = i32;
+pub type pid_t = u32;
 
 use alloc::string::String;
 use alloc::sync::Arc;
 use core::mem::{size_of, size_of_val, transmute, zeroed};
 use core::ops::Deref;
 use core::ptr::{null, null_mut};
-use core::slice::{from_raw_parts, from_raw_parts_mut};
 
 use ntapi::ntpsapi::PROCESS_BASIC_INFORMATION;
 use winapi::shared::minwindef::*;
 use winapi::shared::ntdef::UNICODE_STRING;
-use winapi::shared::windef::*;
 use winapi::um::handleapi::*;
 use winapi::um::memoryapi::*;
 use winapi::um::processthreadsapi::*;
@@ -32,10 +33,9 @@ use winapi::um::psapi::*;
 use winapi::um::tlhelp32::*;
 use winapi::um::winbase::*;
 use winapi::um::winnt::*;
-use winapi::um::winuser::*;
 
 use anyhow::{Error, Result};
-use std::io::Error as IoError;
+use std::io::{Error as IoError, Result as IoResult};
 
 use crate::prelude::*;
 use crate::shell::ProcessInfo;
@@ -153,26 +153,8 @@ impl<T: Copy> Iterator for ToolHelperIter<T> {
     }
 }
 
-#[derive(Serialize, Deserialize)]
-pub struct MemoryPage {
-    pub base: usize,
-    pub alloc_base: usize,
-    pub size: usize,
-    pub type_: u32,
-    pub state: u32,
-    pub protect: u32,
-    pub alloc_protect: u32,
-}
-
-impl crate::range::RangeValue for MemoryPage {
-    #[inline]
-    fn as_range(&self) -> core::ops::Range<usize> {
-        self.base..self.base + self.size
-    }
-}
-
-impl MemoryPage {
-    pub fn from_mbi(mbi: &MEMORY_BASIC_INFORMATION) -> MemoryPage {
+impl From<&MEMORY_BASIC_INFORMATION> for MemoryPage {
+    fn from(mbi: &MEMORY_BASIC_INFORMATION) -> Self {
         MemoryPage {
             base: mbi.BaseAddress as usize,
             alloc_base: mbi.AllocationBase as usize,
@@ -181,67 +163,7 @@ impl MemoryPage {
             state: mbi.State,
             protect: mbi.Protect,
             alloc_protect: mbi.AllocationProtect,
-        }
-    }
-
-    #[inline]
-    pub fn is_commit(&self) -> bool {
-        self.state & MEM_COMMIT > 0
-    }
-
-    #[inline]
-    pub fn is_reserve(&self) -> bool {
-        self.state & MEM_RESERVE > 0
-    }
-
-    #[inline]
-    pub fn is_free(&self) -> bool {
-        self.state & MEM_FREE > 0
-    }
-
-    #[inline]
-    pub fn is_private(&self) -> bool {
-        self.type_ & MEM_PRIVATE > 0
-    }
-
-    pub fn is_executable(&self) -> bool {
-        self.protect & 0xF0 > 0
-    }
-    pub fn is_writable(&self) -> bool {
-        self.protect & 0xCC > 0
-    }
-    pub fn is_readonly(&self) -> bool {
-        self.protect == PAGE_READONLY
-    }
-
-    pub fn protect(&self) -> String {
-        let guard = self.protect & PAGE_GUARD > 0;
-        let mut result = match self.protect & !PAGE_GUARD {
-            PAGE_NOACCESS => "-----",
-            PAGE_READONLY => "-R---",
-            PAGE_READWRITE => "-RW--",
-            PAGE_WRITECOPY => "-RWC-",
-            PAGE_EXECUTE => "E----",
-            PAGE_EXECUTE_READ => "ER---",
-            PAGE_EXECUTE_READWRITE => "ERW--",
-            PAGE_EXECUTE_WRITECOPY => "ERWC-",
-            _ => "?????",
-        }
-        .to_string();
-        if guard {
-            unsafe {
-                result.as_bytes_mut()[4] = b'G';
-            }
-        }
-        result
-    }
-
-    pub fn type_(&self) -> &'static str {
-        match self.type_ {
-            MEM_PRIVATE => "PRV",
-            MEM_IMAGE => "IMG",
-            MEM_MAPPED => "MAP",
-            _ => "",
+            ..Default::default()
         }
     }
 }
@@ -258,19 +180,6 @@ pub impl THREADENTRY32 {
     }
 }
 
-pub fn enum_thread() -> ToolHelperIter<THREADENTRY32> {
-    unsafe {
-        let mut te32: THREADENTRY32 = zeroed();
-        te32.dwSize = size_of_val(&te32) as u32;
-        ToolHelperIter::new(
-            CreateToolhelp32Snapshot(TH32CS_SNAPTHREAD, 0).into(),
-            te32,
-            Thread32First,
-            Thread32Next,
-        )
-    }
-}
-
 #[extend::ext(name = ModuleInfo)]
 pub impl MODULEENTRY32W {
     #[inline(always)]
@@ -355,52 +264,20 @@ pub fn enum_module(pid: u32) -> ToolHelperIter<MODULEENTRY32W> {
     }
 }
 
-pub trait UnicodeUtil {
-    fn size(&self) -> u16;
-    fn max_size(&self) -> u16;
-    fn to_string(&self) -> String;
-    fn as_slice(&self) -> Option<&[u16]>;
-    fn as_mut_slice(&mut self) -> Option<&mut [u16]>;
-    fn as_slice_with_null(&self) -> Option<&[u16]>;
-}
-
-impl UnicodeUtil for UNICODE_STRING {
-    fn size(&self) -> u16 {
-        self.Length
-    }
-    fn max_size(&self) -> u16 {
-        self.MaximumLength
-    }
-
-    fn to_string(&self) -> String {
-        unsafe { from_raw_parts(self.Buffer, self.Length as usize).to_utf8() }
-    }
-
-    fn as_mut_slice(&mut self) -> Option<&mut [u16]> {
-        unsafe {
-            if self.Buffer.is_null() {
-                return None;
-            }
-            Some(from_raw_parts_mut(self.Buffer, self.size() as usize))
-        }
-    }
-
-    fn as_slice(&self) -> Option<&[u16]> {
-        if self.Buffer.is_null() {
-            return None;
-        }
-        Some(unsafe { from_raw_parts(self.Buffer, self.size() as usize) })
-    }
-
-    fn as_slice_with_null(&self) -> Option<&[u16]> {
-        if self.Buffer.is_null() {
-            return None;
-        }
-        Some(unsafe { from_raw_parts(self.Buffer, self.size() as usize + 1) })
+pub fn enum_thread() -> ToolHelperIter<THREADENTRY32> {
+    unsafe {
+        let mut te32: THREADENTRY32 = zeroed();
+        te32.dwSize = size_of_val(&te32) as u32;
+        ToolHelperIter::new(
+            CreateToolhelp32Snapshot(TH32CS_SNAPTHREAD, 0).into(),
+            te32,
+            Thread32First,
+            Thread32Next,
+        )
     }
 }
 
-#[derive(Clone, Copy)]
+#[derive(Debug, Default, Clone, Copy)]
 #[repr(C)]
 pub struct ExceptionRecord {
     pub code: u32,
@@ -521,6 +398,14 @@ impl WriteMemory for Process {
             None
         }
     }
+
+    fn flush_cache(&self, address: usize, len: usize) -> IoResult<()> {
+        if unsafe { FlushInstructionCache(*self.handle, address as LPCVOID, len) > 0 } {
+            Ok(())
+        } else {
+            Err(IoError::last_os_error())
+        }
+    }
 }
 
 impl Process {
@@ -694,14 +579,14 @@ impl Process {
     }
 
     /// Wrapper of QueryFullProcessImageNameW
-    pub fn image_path(&self) -> Option<String> {
+    pub fn image_path(&self) -> UDbgResult<String> {
         unsafe {
             let mut path = [0 as u16; MAX_PATH];
             let mut size = path.len() as u32;
             if QueryFullProcessImageNameW(*self.handle, 0, path.as_mut_ptr(), &mut size) > 0 {
-                Some(path.as_ref().to_utf8())
+                Ok(path.as_ref().to_utf8())
             } else {
-                None
+                Err(UDbgError::system())
             }
         }
     }
@@ -715,7 +600,7 @@ impl Process {
                 self.read_value::<usize>(peb as usize + FIELD_OFFSET!(PEB, ProcessParameters))
             })
             .and_then(|p| {
-                self.read_unicode_string(
+                self.read_value::<UNICODE_STRING>(
                     p + FIELD_OFFSET!(RTL_USER_PROCESS_PARAMETERS, CommandLine),
                 )
             })
@@ -738,15 +623,6 @@ impl Process {
         write_process_memory(*self.handle, address, data)
     }
 
-    pub fn write_code(&self, address: usize, data: &[u8]) -> Result<usize, IoError> {
-        let r = write_process_memory(*self.handle, address, data);
-        if unsafe { FlushInstructionCache(*self.handle, address as LPCVOID, data.len()) > 0 } {
-            Ok(r)
-        } else {
-            Err(IoError::last_os_error())
-        }
-    }
-
     pub fn enum_memory(&self, address: usize) -> MemoryIter {
         MemoryIter {
             process: self,
@@ -767,7 +643,7 @@ impl Process {
             let mut mbi: MEMORY_BASIC_INFORMATION = zeroed();
             match VirtualQueryEx(*self.handle, address as LPVOID, &mut mbi, size_of_val(&mbi)) {
                 0 => None,
-                _ => Some(MemoryPage::from_mbi(&mbi)),
+                _ => Some(MemoryPage::from(&mbi)),
             }
         }
     }
@@ -809,85 +685,51 @@ impl Process {
     }
 }
 
-#[inline(always)]
-fn set_bit(n: &mut reg_t, x: usize, set: bool) {
-    if set {
-        *n |= 1 << x;
-    } else {
-        *n &= !(1 << x);
+impl ReadValue<String> for UNICODE_STRING {
+    fn read_value<R: ReadMemoryUtils + ?Sized>(r: &R, address: usize) -> Option<String> {
+        r.read_copy::<UNICODE_STRING>(address)
+            .and_then(|u| r.read_wstring(u.Buffer as usize, u.Length as usize / 2))
     }
 }
 
-#[inline(always)]
-fn test_bit(n: reg_t, x: usize) -> bool {
-    n & (1 << x) > 0
+impl ReadValue for IMAGE_DOS_HEADER {
+    fn read_value<R: ReadMemoryUtils + ?Sized>(r: &R, address: usize) -> Option<Self> {
+        let dos = r.read_copy::<Self>(address)?;
+        if dos.e_magic != IMAGE_DOS_SIGNATURE {
+            return None;
+        }
+        Some(dos)
+    }
 }
 
-#[inline(always)]
-fn set_bit2(n: &mut reg_t, x: usize, v: reg_t) {
-    *n &= !(0b11 << x);
-    *n |= v << x;
+impl ReadValue for IMAGE_NT_HEADERS {
+    fn read_value<R: ReadMemoryUtils + ?Sized>(r: &R, address: usize) -> Option<Self> {
+        let nt = r.read_copy::<Self>(address)?;
+        if nt.Signature != IMAGE_NT_SIGNATURE {
+            return None;
+        }
+        Some(nt)
+    }
 }
 
-const L0: usize = 0;
-const G0: usize = 1;
-const L1: usize = 2;
-const G1: usize = 3;
-const L2: usize = 4;
-const G2: usize = 5;
-const L3: usize = 5;
-const G3: usize = 7;
-const L_ENABLE: usize = 8;
-const G_ENABLE: usize = 9;
-const RW0: usize = 16;
-const LEN0: usize = 18;
-const RW1: usize = 20;
-const LEN1: usize = 22;
-const RW2: usize = 24;
-const LEN2: usize = 26;
-const RW3: usize = 28;
-const LEN3: usize = 30;
-
 pub trait ReadMemUtilsWin: ReadMemoryUtils {
     fn read_ansi(&self, address: usize, max: impl Into<Option<usize>>) -> Option<String> {
         let r = self.read_cstring(address, max)?;
         Some(r.to_unicode().to_utf8())
     }
 
+    // read a C string, if it is not a valid utf8 string, try convert from ANSI encoding
     fn read_utf8_or_ansi(&self, address: usize, max: impl Into<Option<usize>>) -> Option<String> {
         let r = self.read_cstring(address, max)?;
-        match core::str::from_utf8(&r) {
-            Ok(_) => String::from_utf8(r).ok(),
-            Err(_) => Some(r.to_unicode().to_utf8()),
+        match String::from_utf8(r) {
+            Ok(res) => Some(res),
+            Err(err) => Some(err.as_bytes().to_unicode().to_utf8()),
         }
     }
 
-    fn read_wstring(&self, address: usize, max: impl Into<Option<usize>>) -> Option<String> {
-        let result = self.read_util(
-            address,
-            |&x| x < b' ' as u16 && x != 9 && x != 10 && x != 13,
-            max.into().unwrap_or(1000),
-        );
-        if result.len() == 0 {
-            return None;
-        }
-        Some(result.to_utf8())
-    }
-
-    fn read_unicode_string(&self, address: usize) -> Option<String> {
-        self.read_value::<UNICODE_STRING>(address)
-            .and_then(|u| self.read_wstring(u.Buffer as usize, u.Length as usize / 2))
-    }
-
     fn read_nt_header(&self, mod_base: usize) -> Option<(IMAGE_NT_HEADERS, usize)> {
-        let dos: IMAGE_DOS_HEADER = self.read_value(mod_base)?;
-        if dos.e_magic != IMAGE_DOS_SIGNATURE {
-            return None;
-        }
-        let nt: IMAGE_NT_HEADERS = self.read_value(mod_base + dos.e_lfanew as usize)?;
-        if nt.Signature != IMAGE_NT_SIGNATURE {
-            return None;
-        }
+        let dos = self.read_value::<IMAGE_DOS_HEADER>(mod_base)?;
+        let nt = self.read_value::<IMAGE_NT_HEADERS>(mod_base + dos.e_lfanew as usize)?;
         Some((nt, dos.e_lfanew as usize))
     }
 }
@@ -1002,16 +844,15 @@ pub fn create_debug_process(
 }
 
 impl ProcessInfo {
-    pub fn enumerate() -> Box<dyn Iterator<Item = ProcessInfo>> {
+    pub fn enumerate() -> UDbgResult<impl Iterator<Item = ProcessInfo>> {
         use winapi::um::winnt::*;
 
-        Box::new(enum_process().map(|p| {
+        Ok(enum_process().map(|p| {
             let pid = p.pid();
             let mut result = ProcessInfo {
                 pid,
                 name: p.name(),
                 wow64: false,
-                // window: get_window(pid).map(|w| w.get_text()).unwrap_or(String::new()),
                 path: String::new(),
                 cmdline: String::new(),
             };
