@@ -2,91 +2,65 @@
 //! Traits && types for debugger target, such as memory page, module, thread, process, etc., and their iterators.
 //!
 
+use crate::os::{priority_t, Module, Process};
+use crate::{pe::*, prelude::*, register::*};
+
 use core::ops::Deref;
+use parking_lot::RwLock;
+use std::borrow::Cow;
 use std::cell::Cell;
-use std::io::{Error as IoError, ErrorKind, Result as IoResult};
+use std::collections::HashMap;
+use std::io::{ErrorKind, Result as IoResult};
 use std::sync::Arc;
 
-use crate::{
-    os::{pid_t, tid_t},
-    prelude::*,
-    register::*,
-};
-
-#[derive(Copy, Clone, Debug, PartialEq)]
+#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
 pub enum UDbgStatus {
-    Idle,
     Opened,
     Attached,
-    Paused,
-    Running,
-    Ended,
+    Detaching,
+    Detached,
 }
 
-#[derive(Clone)]
-pub struct PauseContext {
-    pub arch: Cell<u32>,
-    pub psize: Cell<usize>,
-}
-
-impl Default for PauseContext {
-    fn default() -> Self {
-        Self {
-            arch: UDBG_ARCH.into(),
-            psize: core::mem::size_of::<usize>().into(),
+impl UDbgStatus {
+    pub fn as_str(&self) -> &'static str {
+        match self {
+            UDbgStatus::Opened => "opened",
+            UDbgStatus::Attached => "attached",
+            UDbgStatus::Detaching => "detaching",
+            UDbgStatus::Detached => "detached",
         }
     }
 }
 
-impl PauseContext {
-    pub fn update(&self, arch: u32) {
-        self.arch.set(arch);
-        match arch {
-            ARCH_X86 | ARCH_ARM => self.psize.set(4),
-            ARCH_X64 | ARCH_ARM64 => self.psize.set(8),
-            _ => {}
-        };
-    }
-}
-
 /// Common data for debugger target
 #[derive(Clone, Serialize)]
 pub struct TargetBase {
-    /// process id of target
+    /// Process ID of target, if target is a process
     pub pid: Cell<pid_t>,
-    /// thread id of target triggers the debug event
+    /// Thread ID of target triggers the debug event
     pub event_tid: Cell<tid_t>,
-    pub event_pc: Cell<usize>,
-    /// executable image path of target
-    pub image_path: String,
+    /// Module base address of executable image
     pub image_base: usize,
-    /// architecture of target, one value of the [std::env::consts::ARCH]
+    /// Architecture of target, one value of the [std::env::consts::ARCH]
     pub arch: &'static str,
-    /// if the target is a [WOW64](https://docs.microsoft.com/en-us/windows/win32/winprog64/running-32-bit-applications) process
-    #[cfg(windows)]
-    pub wow64: Cell<bool>,
+    /// Context architecture when target interruptted
+    pub context_arch: Cell<u32>,
     #[serde(skip)]
     pub flags: Cell<UDbgFlags>,
     #[serde(skip)]
-    pub context: PauseContext,
-    #[serde(skip)]
     pub status: Cell<UDbgStatus>,
 }
 
 impl Default for TargetBase {
     fn default() -> Self {
         Self {
-            image_path: "".into(),
             image_base: 0,
-            context: Default::default(),
-            event_pc: Default::default(),
             event_tid: Default::default(),
             pid: Cell::new(0),
             flags: Default::default(),
-            #[cfg(windows)]
-            wow64: Cell::new(false),
             arch: std::env::consts::ARCH,
-            status: Cell::new(UDbgStatus::Attached),
+            context_arch: Cell::new(UDBG_ARCH),
+            status: Cell::new(UDbgStatus::Opened),
         }
     }
 }
@@ -94,41 +68,102 @@ impl Default for TargetBase {
 impl TargetBase {
     #[inline]
     pub fn is_ptr32(&self) -> bool {
-        self.ptrsize() == 4
+        self.pointer_size() == 4
+    }
+
+    /// if the target is a [WOW64](https://docs.microsoft.com/en-us/windows/win32/winprog64/running-32-bit-applications) process
+    #[inline]
+    pub fn is_wow64(&self) -> bool {
+        self.context_arch.get() == ARCH_X86
     }
 
     #[inline]
-    pub fn ptrsize(&self) -> usize {
-        self.context.psize.get()
-    }
-
-    pub fn update_arch(&self, arch: u32) {
-        if arch == self.context.arch.get() {
-            return;
+    pub fn pointer_size(&self) -> usize {
+        match self.context_arch.get() {
+            ARCH_X86 | ARCH_ARM => 4,
+            _ => core::mem::size_of::<usize>(),
         }
-        self.context.update(arch);
-        udbg_ui().update_arch(arch);
     }
 
-    pub fn is_opened(&self) -> bool {
-        self.status.get() == UDbgStatus::Opened
-    }
-
-    pub fn is_paused(&self) -> bool {
-        self.status.get() == UDbgStatus::Paused
-    }
-
-    pub fn check_opened(&self) -> UDbgResult<()> {
-        if self.is_opened() {
-            Err(UDbgError::NotSupport)
+    pub fn check_attached(&self) -> UDbgResult<()> {
+        if self.status.get() < UDbgStatus::Attached {
+            Err(UDbgError::NotAttached)
         } else {
             Ok(())
         }
     }
+}
+
+#[derive(Deref, DerefMut)]
+pub struct CommonBase {
+    #[deref]
+    #[deref_mut]
+    pub base: TargetBase,
+    pub process: Process,
+    pub step_tid: Cell<tid_t>,
+    pub symgr: SymbolManager<Module>,
+    pub bp_map: RwLock<HashMap<BpID, Arc<Breakpoint>>>,
+    pub dbg_reg: [Cell<usize>; 4],
+}
+
+impl CommonBase {
+    pub fn new(ps: Process) -> Self {
+        let base = TargetBase::default();
+        base.pid.set(ps.pid());
+        Self {
+            base,
+            process: ps,
+            step_tid: Cell::new(0),
+            symgr: Default::default(),
+            dbg_reg: Default::default(),
+            bp_map: RwLock::new(HashMap::new()),
+        }
+    }
+
+    pub fn get_hwbp_index(&self) -> Option<usize> {
+        for (i, p) in self.dbg_reg.iter().enumerate() {
+            if p.get() == 0 {
+                return Some(i);
+            }
+        }
+        None
+    }
+
+    pub fn set_hwbp(&self, index: usize, p: usize) {
+        self.dbg_reg.get(index).map(|cell| cell.set(p));
+    }
+
+    #[inline]
+    pub fn enable_hwbp_for_context<C: HWBPRegs>(&self, cx: &mut C, info: HwbpInfo, enable: bool) {
+        let i = info.index as usize;
+        if enable {
+            cx.set_bp(self.dbg_reg[i].get(), i, info.rw, info.len);
+        } else {
+            cx.unset_bp(i);
+        }
+    }
+
+    pub fn find_table_bp_index(&self) -> Option<isize> {
+        for i in -10000..-10 {
+            if self.bp_map.read().get(&i).is_none() {
+                return Some(i);
+            }
+        }
+        None
+    }
 
     #[inline(always)]
-    pub fn undec_sym(&self, sym: &str) -> Option<String> {
-        Symbol::undecorate(sym, self.flags.get())
+    pub fn find_module(&self, address: usize) -> Option<Arc<Module>> {
+        self.symgr.find_module(address)
+    }
+
+    #[inline(always)]
+    pub fn bp_exists(&self, id: BpID) -> bool {
+        self.bp_map.read().get(&id).is_some()
+    }
+
+    pub fn get_bp<'a>(&'a self, id: BpID) -> Option<Arc<dyn UDbgBreakpoint + 'a>> {
+        Some(self.bp_map.read().get(&id)?.clone())
     }
 }
 
@@ -139,7 +174,7 @@ pub struct ThreadData {
     #[cfg(windows)]
     pub handle: crate::os::windows::Handle,
     #[cfg(target_os = "macos")]
-    pub handle: crate::process::ThreadAct,
+    pub handle: crate::os::macos::ThreadAct,
 }
 
 #[cfg(windows)]
@@ -147,46 +182,52 @@ pub type ThreadContext = winapi::um::winnt::CONTEXT;
 #[cfg(windows)]
 pub type ThreadContext32 = super::register::CONTEXT32;
 
+/// Represents a thread in target
 pub trait UDbgThread: Deref<Target = ThreadData> + GetProp {
+    /// Thread name
     fn name(&self) -> Arc<str> {
         "".into()
     }
+
     fn status(&self) -> Arc<str> {
         "".into()
     }
 
-    /// see https://docs.microsoft.com/en-us/windows/win32/api/processthreadsapi/nf-processthreadsapi-getthreadpriority#return-value
-    #[cfg(windows)]
-    fn priority(&self) -> Option<i32> {
+    /// Get thread's priority
+    fn priority(&self) -> Option<priority_t> {
         None
     }
 
-    #[cfg(not(windows))]
-    fn priority(&self) -> Arc<str> {
-        "".into()
-    }
-
+    /// Suspend the thread, and return the suspend count if success
     fn suspend(&self) -> IoResult<i32> {
         Err(ErrorKind::Unsupported.into())
     }
+
+    /// Resume the thread, and return the suspend count if success
     fn resume(&self) -> IoResult<u32> {
         Err(ErrorKind::Unsupported.into())
     }
+
+    /// Get thread's suspend count
+    fn suspend_count(&self) -> usize {
+        0
+    }
+
     #[cfg(windows)]
     fn get_context(&self, cx: &mut ThreadContext) -> IoResult<()> {
-        Err(IoError::from(ErrorKind::Unsupported))
+        Err(ErrorKind::Unsupported.into())
     }
     #[cfg(windows)]
     fn set_context(&self, cx: &ThreadContext) -> IoResult<()> {
-        Err(IoError::from(ErrorKind::Unsupported))
+        Err(ErrorKind::Unsupported.into())
     }
     #[cfg(windows)]
     fn get_context32(&self, cx: &mut ThreadContext32) -> IoResult<()> {
-        Err(IoError::from(ErrorKind::Unsupported))
+        Err(ErrorKind::Unsupported.into())
     }
     #[cfg(windows)]
     fn set_context32(&self, cx: &ThreadContext32) -> IoResult<()> {
-        Err(IoError::from(ErrorKind::Unsupported))
+        Err(ErrorKind::Unsupported.into())
     }
     #[cfg(windows)]
     fn teb(&self) -> Option<usize> {
@@ -202,55 +243,68 @@ pub trait UDbgThread: Deref<Target = ThreadData> + GetProp {
     }
 }
 
-/// Debugger Engine
+impl core::fmt::Debug for dyn UDbgThread {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+        let mut ds = f.debug_struct("UDbgThread");
+        ds.field("tid", &self.tid)
+            .field("name", &self.name())
+            .field("status", &self.status())
+            .field("priority", &self.priority())
+            .field("suspend_count", &self.suspend_count());
+        #[cfg(windows)]
+        ds.field("entry", &self.entry()).field("teb", &self.teb());
+        ds.finish()
+    }
+}
+
+/// Debugger Engine, interfaces of debugging
 pub trait UDbgEngine {
-    fn enum_process(&self) -> Box<dyn Iterator<Item = ProcessInfo>> {
-        ProcessInfo::enumerate()
+    fn enum_process(&self) -> UDbgResult<Box<dyn Iterator<Item = ProcessInfo>>> {
+        Ok(Box::new(ProcessInfo::enumerate()?))
     }
 
-    /// open a process, only open, not attach
-    fn open(&mut self, pid: pid_t) -> UDbgResult<Arc<dyn UDbgAdaptor>> {
+    /// Open a process, not attach, for non-invasive debugging purpose
+    fn open(&mut self, pid: pid_t) -> UDbgResult<Arc<dyn UDbgTarget>> {
         Err(UDbgError::NotSupport)
     }
 
-    fn open_self(&mut self) -> UDbgResult<Arc<dyn UDbgAdaptor>> {
+    fn open_self(&mut self) -> UDbgResult<Arc<dyn UDbgTarget>> {
         self.open(std::process::id() as _)
     }
 
-    /// attach to a active process
-    fn attach(&mut self, pid: pid_t) -> UDbgResult<Arc<dyn UDbgAdaptor>>;
+    /// Attach to a active process
+    fn attach(&mut self, pid: pid_t) -> UDbgResult<Arc<dyn UDbgTarget>>;
 
-    /// create and debug a process
+    /// Create and debug a process
     fn create(
         &mut self,
         path: &str,
         cwd: Option<&str>,
         args: &[&str],
-    ) -> UDbgResult<Arc<dyn UDbgAdaptor>>;
-
-    fn do_cmd(&self, cmd: &str) -> UDbgResult<()> {
-        Err(UDbgError::NotSupport)
-    }
+    ) -> UDbgResult<Arc<dyn UDbgTarget>>;
 
+    /// Start the debug event loop, with a event callback
     fn event_loop<'a>(&mut self, callback: &mut UDbgCallback<'a>) -> UDbgResult<()> {
         Err(UDbgError::NotSupport)
     }
 
-    // fn loop_event<
-    //     U: std::future::Future<Output = ()> + 'static,
-    //     F: FnOnce(Arc<Self>, UEventState) -> U,
-    // >(
-    //     self: Arc<Self>,
-    //     callback: F,
-    // ) {
-    //     let state = UEventState::new();
-    //     let mut fetcher = ReplyFetcher::new(Box::pin(callback(self.clone(), state.clone())), state);
-    //     self.event_loop(&mut |event| fetcher.fetch(event).unwrap_or(UserReply::Run(false)));
-    //     fetcher.fetch(None);
-    // }
+    /// Start the debug event loop, wraps a async task as event callback
+    fn task_loop(&mut self, mut task: DebugTask) -> UDbgResult<()> {
+        self.event_loop(&mut |ctx, event| {
+            task.state.ctx.set(unsafe { core::mem::transmute(ctx) });
+            match task.run_step(event) {
+                Some(reply) => reply,
+                // None if ef.ended => {
+                //     // TODO: kill all
+                //     panic!("")
+                // }
+                None => UserReply::Run(false),
+            }
+        })
+    }
 }
 
-pub type UDbgCallback<'a> = dyn FnMut(Arc<dyn UDbgAdaptor>, UEvent) -> UserReply + 'a;
+pub type UDbgCallback<'a> = dyn FnMut(&mut dyn TraceContext, UEvent) -> UserReply + 'a;
 
 /// Trait for getting property dynamically
 pub trait GetProp {
@@ -277,13 +331,27 @@ pub trait TargetControl {
     fn resume(&self) -> UDbgResult<()> {
         Err(UDbgError::NotSupport)
     }
+    /// wait for target to exit
+    fn wait(&self) -> UDbgResult<u32> {
+        Err(UDbgError::NotSupport)
+    }
 }
 
 /// Represent a debugable target, could be a process, core dump, etc.
-pub trait Target: GetProp + TargetMemory + TargetControl + BreakpointManager {
+pub trait Target: GetProp + TargetMemory + TargetControl {
     fn base(&self) -> &TargetBase;
 
-    /// the process handle, if target is a process
+    /// Return the reference of processs if self is a process
+    fn process(&self) -> Option<&Process> {
+        None
+    }
+
+    /// Executable image path of target
+    fn image_path(&self) -> UDbgResult<String> {
+        Ok(self.process().ok_or(UDbgError::NoTarget)?.image_path()?)
+    }
+
+    /// Raw process handle, if target is a process
     #[cfg(windows)]
     fn handle(&self) -> winapi::um::winnt::HANDLE {
         core::ptr::null_mut()
@@ -351,7 +419,7 @@ pub trait Target: GetProp + TargetMemory + TargetControl + BreakpointManager {
                 if soffset > max_offset { return None; }
                 Some(SymbolInfo {
                     mod_base: d.base, offset: soffset, module: d.name.clone(),
-                    symbol: if let Some(n) = self.base().undec_sym(s.name.as_ref()) { n.into() } else { s.name }
+                    symbol: if let Some(n) = Symbol::undecorate(s.name.as_ref(), self.base().flags.get()) { n.into() } else { s.name }
                 })
             }).or_else(|| /* if let Some((b, e, _)) = m.find_function(offset) {
                 Some(SymbolInfo { mod_base: d.base, offset: offset - b as usize, module: d.name.clone(), symbol: format!("${:x}", d.base + b as usize).into() })
@@ -366,17 +434,25 @@ pub trait Target: GetProp + TargetMemory + TargetControl + BreakpointManager {
     }
 }
 
-pub trait UDbgAdaptor: Send + Sync + Target + 'static {
-    fn get_registers(&self) -> UDbgResult<&mut dyn UDbgRegs> {
-        Err(UDbgError::NotSupport)
-    }
+/// Represent a debugable target, which is used in udbg
+pub trait UDbgTarget: Send + Sync + Target + BreakpointManager + 'static {}
 
-    fn except_param(&self, i: usize) -> Option<usize> {
-        None
+impl core::fmt::Debug for dyn UDbgTarget {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+        let base = self.base();
+        f.debug_struct("UDbgTarget")
+            .field("pid", &base.pid.get())
+            .field("image_path", &self.image_path())
+            .finish()
     }
 }
 
-pub trait AdaptorUtil: UDbgAdaptor {
+/// Practical functions based on [`UDbgTarget`]
+pub trait TargetUtil: UDbgTarget {
+    fn add_bp(&self, opt: impl Into<BpOpt>) -> UDbgResult<Arc<dyn UDbgBreakpoint>> {
+        self.add_breakpoint(opt.into())
+    }
+
     fn read_ptr(&self, a: usize) -> Option<usize> {
         if self.base().is_ptr32() {
             self.read_value::<u32>(a).map(|r| r as usize)
@@ -393,42 +469,18 @@ pub trait AdaptorUtil: UDbgAdaptor {
         }
     }
 
-    fn get_reg(&self, r: &str) -> UDbgResult<CpuReg> {
-        self.get_registers()?
-            .get_reg(get_regid(r).ok_or(UDbgError::InvalidRegister)?)
-            .ok_or(UDbgError::InvalidRegister)
-    }
-
-    fn set_reg(&self, r: &str, val: CpuReg) -> UDbgResult<()> {
-        self.get_registers()?
-            .set_reg(get_regid(r).ok_or(UDbgError::InvalidRegister)?, val);
-        Ok(())
-    }
-
-    fn parse_address(&self, symbol: &str) -> Option<usize> {
-        let (mut left, right) = match symbol.find('+') {
-            Some(pos) => ((&symbol[..pos]).trim(), Some((&symbol[pos + 1..]).trim())),
-            None => (symbol.trim(), None),
-        };
-
-        let mut val = if let Ok(val) = self.get_reg(left) {
-            val.as_int()
-        } else {
-            if left.starts_with("0x") || left.starts_with("0X") {
-                left = &left[2..];
-            }
-            if let Ok(address) = usize::from_str_radix(left, 16) {
-                address
-            } else {
-                self.get_address_by_symbol(left)?
-            }
-        };
-
-        if let Some(right) = right {
-            val += self.parse_address(right)?;
+    fn read_argument(
+        &self,
+        reg: &dyn UDbgRegs,
+        i: usize,
+        cc: Option<CallingConv>,
+    ) -> Option<usize> {
+        match reg.argument(i, cc) {
+            Ok(id) => Some(reg.get_reg(id)?.as_int()),
+            Err(n) => self.read_ptr(
+                reg.get_reg(regid::COMM_REG_SP)?.as_int() + n * self.base().pointer_size(),
+            ),
         }
-
-        Some(val)
     }
 
     fn get_symbol_(&self, addr: usize, o: Option<usize>) -> Option<SymbolInfo> {
@@ -456,20 +508,10 @@ pub trait AdaptorUtil: UDbgAdaptor {
         if base.image_base > 0 {
             self.find_module(base.image_base)
         } else {
-            let image_path = &self.base().image_path;
+            let image_base = self.base().image_base;
             for m in self.enum_module().ok()? {
-                let path = m.data().path.clone();
-                #[cfg(windows)]
-                {
-                    if path.eq_ignore_ascii_case(&image_path) {
-                        return Some(m);
-                    }
-                }
-                #[cfg(not(windows))]
-                {
-                    if path.as_ref() == image_path {
-                        return Some(m);
-                    }
+                if image_base == m.data().base {
+                    return Some(m);
                 }
             }
             None
@@ -534,10 +576,10 @@ pub trait AdaptorUtil: UDbgAdaptor {
         self.base().pid.get()
     }
 }
-impl<'a, T: UDbgAdaptor + ?Sized + 'a> AdaptorUtil for T {}
+impl<'a, T: UDbgTarget + ?Sized + 'a> TargetUtil for T {}
 
 #[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
-pub trait AdaptorArchUtil: UDbgAdaptor {
+pub trait TargetArchUtil: UDbgTarget {
     fn disasm(&self, address: usize) -> Option<iced_x86::Instruction> {
         use iced_x86::{Decoder, DecoderOptions, Instruction};
 
@@ -571,11 +613,146 @@ pub trait AdaptorArchUtil: UDbgAdaptor {
 }
 
 #[cfg(any(target_arch = "arm", target_arch = "aarch64"))]
-pub trait AdaptorArchUtil: UDbgAdaptor {
+pub trait TargetArchUtil: UDbgTarget {
     #[inline(always)]
     fn check_call(&self, address: usize) -> Option<usize> {
         todo!();
     }
 }
 
-impl<'a, T: UDbgAdaptor + ?Sized + 'a> AdaptorArchUtil for T {}
+impl<'a, T: UDbgTarget + ?Sized + 'a> TargetArchUtil for T {}
+
+/// Context information during the debugging target interruption
+pub trait TraceContext {
+    /// Registers of debugging thread
+    fn register(&mut self) -> Option<&mut dyn UDbgRegs>;
+    /// Interruptted debugging target
+    fn target(&self) -> Arc<dyn UDbgTarget>;
+    /// Parameter of exception
+    fn exception_param(&self, i: usize) -> Option<usize> {
+        None
+    }
+
+    /// Context architecture, used for WOW64 debugging. Returned value is one of udbg::consts::ARCH_*
+    fn arch(&self) -> u32 {
+        crate::consts::UDBG_ARCH
+    }
+
+    fn pointer_size(&self) -> usize {
+        match self.arch() {
+            ARCH_X86 | ARCH_ARM => 4,
+            _ => core::mem::size_of::<usize>(),
+        }
+    }
+}
+
+impl MemoryPage {
+    #[inline]
+    pub fn is_commit(&self) -> bool {
+        if self.is_windows() {
+            self.state & MEM_COMMIT > 0
+        } else {
+            true
+        }
+    }
+
+    #[inline]
+    pub fn is_reserve(&self) -> bool {
+        if self.is_windows() {
+            self.state & MEM_RESERVE > 0
+        } else {
+            false
+        }
+    }
+
+    #[inline]
+    pub fn is_free(&self) -> bool {
+        if self.is_windows() {
+            self.state & MEM_FREE > 0
+        } else {
+            false
+        }
+    }
+
+    pub fn protect(&self) -> Cow<str> {
+        if self.is_windows() {
+            let result = match self.protect & !PAGE_GUARD {
+                PAGE_NOACCESS => "-----",
+                PAGE_READONLY => "-R---",
+                PAGE_READWRITE => "-RW--",
+                PAGE_WRITECOPY => "-RWC-",
+                PAGE_EXECUTE => "E----",
+                PAGE_EXECUTE_READ => "ER---",
+                PAGE_EXECUTE_READWRITE => "ERW--",
+                PAGE_EXECUTE_WRITECOPY => "ERWC-",
+                _ => "?????",
+            };
+            if self.protect & PAGE_GUARD > 0 {
+                let mut res = result.to_string();
+                unsafe {
+                    res.as_bytes_mut()[4] = b'G';
+                }
+                res.into()
+            } else {
+                result.into()
+            }
+        } else {
+            unsafe { core::str::from_utf8_unchecked(self.as_linux_protect()) }.into()
+        }
+    }
+
+    pub fn type_(&self) -> &'static str {
+        if self.is_windows() {
+            match self.type_ {
+                MEM_PRIVATE => "PRV",
+                MEM_IMAGE => "IMG",
+                MEM_MAPPED => "MAP",
+                _ => "",
+            }
+        } else if self.is_private() {
+            "PRV"
+        } else if self.is_shared() {
+            "SHR"
+        } else {
+            ""
+        }
+    }
+
+    #[inline]
+    pub fn is_private(&self) -> bool {
+        if self.is_windows() {
+            self.type_ & MEM_PRIVATE > 0
+        } else {
+            self.as_linux_protect()[3] == b'p'
+        }
+    }
+
+    #[inline]
+    pub fn is_shared(&self) -> bool {
+        self.as_linux_protect()[3] == b's'
+    }
+
+    pub fn is_executable(&self) -> bool {
+        if self.is_windows() {
+            self.protect & 0xF0 > 0
+        } else {
+            self.as_linux_protect()[2] == b'x'
+        }
+    }
+
+    pub fn is_writable(&self) -> bool {
+        if self.is_windows() {
+            self.protect & 0xCC > 0
+        } else {
+            self.as_linux_protect()[1] == b'w'
+        }
+    }
+
+    pub fn is_readonly(&self) -> bool {
+        if self.is_windows() {
+            self.protect == PAGE_READONLY
+        } else {
+            self.as_linux_protect()[0] == b'r' && !self.is_writable() && !self.is_executable()
+        }
+    }
+}
